@@ -1,13 +1,11 @@
-use crate::mechanism::Kind;
-use crate::mechanism::Mechanism;
-use crate::mechanism::Qualifier;
+use crate::mechanism::{Kind, Mechanism, Qualifier};
 use ipnetwork::IpNetwork;
 use lazy_static::lazy_static;
 use regex::Regex;
 use std::num::ParseIntError;
 
 // Provide domain host validation check.
-#[cfg(feature = "warn-dns")]
+#[cfg(any(feature = "warn-dns", feature = "strict-dns"))]
 use addr::parse_dns_name;
 
 /// This is the maximnum number of characters that an Spf Record can store.
@@ -50,19 +48,14 @@ pub(crate) fn capture_matches(string: &str, kind: Kind) -> Option<Mechanism<Stri
         None => None,
         Some(caps) => {
             // There was a match
-            if caps.name("qualifier").is_some() {
-                qualifier_char = caps
-                    .name("qualifier")
-                    .unwrap()
-                    .as_str()
-                    .chars()
-                    .next()
-                    .unwrap();
+            if let Some(qualifier) = caps.name("qualifier") {
+                qualifier_char = qualifier.as_str().chars().next().unwrap();
                 qualifier_result = char_to_qualifier(qualifier_char);
-            };
-            if caps.name("mechanism").is_some() {
+            }
+            //if caps.name("mechanism").is_some() {
+            if let Some(mechanism_value) = caps.name("mechanism") {
                 let mut new_mechanism: String = String::new();
-                mechanism_string = caps.name("mechanism").unwrap().as_str().to_string();
+                mechanism_string = mechanism_value.as_str().to_string();
                 // Check if we got a match on a number. No more than 3 digits.
                 // Regex matches [ domain, domain/num, num] so we handle the "num" case here
                 if mechanism_string.len() < 4 {
@@ -80,13 +73,13 @@ pub(crate) fn capture_matches(string: &str, kind: Kind) -> Option<Mechanism<Stri
                 if !new_mechanism.is_empty() {
                     mechanism_string = new_mechanism;
                 }
-                mechanism = Mechanism::new(
+                mechanism = Mechanism::generic_inclusive(
                     kind,
                     qualifier_result,
                     Some((*mechanism_string).to_string()),
                 );
             } else {
-                mechanism = Mechanism::new(kind, qualifier_result, None);
+                mechanism = Mechanism::generic_inclusive(kind, qualifier_result, None);
             }
             Some(mechanism)
         }
@@ -112,7 +105,7 @@ pub(crate) fn char_to_qualifier(c: char) -> Qualifier {
 }
 
 // builds a string representation of of the mechanisms stored in the Vec<Mechanism<String>>
-pub(crate) fn build_spf_str(str: &Vec<Mechanism<String>>) -> String {
+pub(crate) fn build_spf_str(str: &[Mechanism<String>]) -> String {
     let mut partial_spf = String::new();
     for i in str.iter() {
         partial_spf.push(' ');
@@ -121,7 +114,7 @@ pub(crate) fn build_spf_str(str: &Vec<Mechanism<String>>) -> String {
     partial_spf
 }
 // builds a string representation of of the mechanisms stored in the Vec<Mechanism<IpNetwork>>
-pub(crate) fn build_spf_str_from_ip(str: &Vec<Mechanism<IpNetwork>>) -> String {
+pub(crate) fn build_spf_str_from_ip(str: &[Mechanism<IpNetwork>]) -> String {
     let mut partial_spf = String::new();
     for i in str.iter() {
         partial_spf.push(' ');
@@ -149,7 +142,7 @@ pub(crate) fn return_and_remove_qualifier(record: &str, c: char) -> (Qualifier,
         (Qualifier::Pass, record)
     }
 }
-#[cfg(feature = "warn-dns")]
+#[cfg(any(feature = "warn-dns", feature = "strict-dns"))]
 pub(crate) fn get_domain_before_slash(s: &str) -> &str {
     if !s.starts_with('/') && s.contains('/') {
         s.split('/').next().unwrap()
@@ -176,12 +169,18 @@ fn domain_without_slash() {
     assert_eq!(get_domain_before_slash(input), "test.com");
 }
 
+#[cfg(feature = "warn-dns")]
+pub(crate) fn check_for_dns_warning(warning_vec: &mut Vec<String>, name: &str) {
+    if !dns_is_valid(name) {
+        warning_vec.push(name.to_string());
+    }
+}
 // Return true if the domain/host is valid.
 #[allow(dead_code)]
-#[cfg(feature = "warn-dns")]
+#[cfg(any(feature = "warn-dns", feature = "strict-dns"))]
 pub(crate) fn dns_is_valid(name: &str) -> bool {
     // These can not be and do not need to be tested. They are always valid.
-    if name == "a" || name == "mx" || name == "ptr" {
+    if name == "a" || name == "mx" || name == "ptr" || name == "all" || name.starts_with('/') {
         true
     } else {
         match parse_dns_name(name) {
