@@ -7,7 +7,7 @@
 //! - [`ParsedMechanism`](parsedmechanism::ParsedMechanism)
 //!     - This provides a unified method for parsing any mechanism string. It will either contain a `Mechanism<String>`
 //! or a `Mechanism<IpNetwork>` if the string is succesfully parsed.
-//! - Both `Mechanism<String>` and `Mechanism<IpNetwork>` have the `FromStr` trait implemented. Allowing the for the strings
+//! - Both `Mechanism<String>` and `Mechanism<IpNetwork>` have the `FromStr` trait implemented. Allowing for the strings
 //! to be `parsed()`
 //! - The `Mechanism` struct also has a number of specfic methods which can be used to create related mechanisms; which are
 //! used with the `FromStr` trait.
@@ -24,7 +24,8 @@ pub use crate::mechanism::parsedmechanism::ParsedMechanism;
 pub use crate::mechanism::qualifier::Qualifier;
 
 use crate::helpers;
-use ipnetwork::IpNetwork;
+use ipnetwork::{IpNetwork, IpNetworkError};
+
 use std::{convert::TryFrom, str::FromStr};
 
 /// Stores its [`Kind`](Kind), [`Qualifier`](Qualifier), and its `Value`
@@ -39,15 +40,17 @@ pub struct Mechanism<T> {
 ///
 /// # Examples:
 ///```rust
-/// use decon_spf::mechanism::Mechanism;
+/// # use decon_spf::mechanism::Mechanism;
 /// let a: Mechanism<String> = "a".parse().unwrap();
 /// assert_eq!(a.kind().is_a(), true);
 ///
-/// let mx = "mx".parse::<Mechanism<String>>().unwrap();
-/// assert_eq!(mx.kind().is_mx(), true);
-/// let mx2 = "-mx:example.com".parse::<Mechanism<String>>().unwrap();
-/// assert_eq!(mx2.qualifier().is_fail(), true);
-/// assert_eq!(mx2.to_string(), "-mx:example.com");
+/// if let Ok(mx) = "mx".parse::<Mechanism<String>>() {
+///   assert_eq!(mx.kind().is_mx(), true);
+/// }
+/// if let Ok(mx2) = "-mx:example.com".parse::<Mechanism<String>>() {
+///   assert_eq!(mx2.qualifier().is_fail(), true);
+///   assert_eq!(mx2.to_string(), "-mx:example.com");
+/// }
 ///
 ///```
 impl FromStr for Mechanism<String> {
@@ -56,44 +59,56 @@ impl FromStr for Mechanism<String> {
     fn from_str(s: &str) -> Result<Mechanism<String>, Self::Err> {
         // A String ending wiith either ':' or "/" is always invalid.
         if s.ends_with(':') || s.ends_with('/') {
-            return Err(MechanismError::NotValidMechanismFormat(s.to_string()));
+            return Err(MechanismError::InvalidMechanismFormat(s.to_string()));
         };
         if s.contains("ip4:") || s.contains("ip6:") {
-            return Err(MechanismError::NotValidMechanismFormat(s.to_string()));
+            return Err(MechanismError::InvalidMechanismFormat(s.to_string()));
         }
+        let mut m: Option<Mechanism<String>> = None;
+
         if s.contains("redirect=") {
             let mut items = s.rsplit('=');
-            if let Some(item) = items.next() {
-                return Ok(Mechanism::new(
+            if let Some(rrdata) = items.next() {
+                m = Some(Mechanism::generic_inclusive(
                     Kind::Redirect,
                     Qualifier::Pass,
-                    Some(item.to_string()),
+                    Some(rrdata.to_string()),
                 ));
             }
         } else if s.contains("include:") {
             let qualifier_and_modified_str = helpers::return_and_remove_qualifier(s, 'i');
-            if let Some(item) = s.rsplit(':').next() {
-                return Ok(Mechanism::new_include(
+            if let Some(rrdata) = s.rsplit(':').next() {
+                m = Some(Mechanism::generic_inclusive(
+                    Kind::Include,
                     qualifier_and_modified_str.0,
-                    item.to_string(),
+                    Some(rrdata.to_string()),
                 ));
             }
         } else if s.ends_with("all") && (s.len() == 3 || s.len() == 4) {
-            return Ok(Mechanism::new_all(
+            m = Some(Mechanism::all(
                 helpers::return_and_remove_qualifier(s, 'a').0,
             ));
         } else if let Some(a_mechanism) = helpers::capture_matches(s, Kind::A) {
-            return Ok(a_mechanism);
+            m = Some(a_mechanism);
         } else if let Some(mx_mechanism) = helpers::capture_matches(s, Kind::MX) {
-            return Ok(mx_mechanism);
+            m = Some(mx_mechanism);
         } else if let Some(ptr_mechanism) = helpers::capture_matches(s, Kind::Ptr) {
-            return Ok(ptr_mechanism);
+            m = Some(ptr_mechanism);
         } else if let Some(exists_mechanism) = helpers::capture_matches(s, Kind::Exists) {
             if !exists_mechanism.raw().contains('/') {
-                return Ok(exists_mechanism);
+                m = Some(exists_mechanism);
             }
         }
-        Err(MechanismError::NotValidMechanismFormat(s.to_string()))
+        if let Some(value) = m {
+            #[cfg(feature = "strict-dns")]
+            {
+                if !helpers::dns_is_valid(helpers::get_domain_before_slash(&value.raw())) {
+                    return Err(MechanismError::InvalidDomainHost(value.raw()));
+                }
+            }
+            return Ok(value);
+        }
+        Err(MechanismError::InvalidMechanismFormat(s.to_string()))
     }
 }
 
@@ -109,8 +124,7 @@ impl TryFrom<&str> for Mechanism<String> {
 ///
 /// # Examples:
 ///```rust
-/// use decon_spf::mechanism::MechanismError;
-/// use decon_spf::mechanism::Mechanism;
+/// # use decon_spf::mechanism::{Mechanism, MechanismError};
 /// # use ipnetwork::IpNetwork;
 /// let ip4: Mechanism<IpNetwork> = "ip4:203.32.160.0/24".parse().unwrap();
 /// assert_eq!(ip4.kind().is_ip_v4(), true);
@@ -119,12 +133,12 @@ impl TryFrom<&str> for Mechanism<String> {
 /// assert_eq!(ip6.kind().is_ip_v6(), true);
 ///
 /// let bad_ip4: Result<Mechanism<IpNetwork>, MechanismError> = "ip4:203.32.160.0/33".parse();
-/// assert_eq!(bad_ip4.unwrap_err().to_string(), "invalid address: 203.32.160.0/33.");
+/// assert_eq!(bad_ip4.unwrap_err().to_string(), "invalid address: 203.32.160.0/33");
 ///
-/// let ip6_but_ip6: Result<Mechanism<IpNetwork>, MechanismError> = "ip6:203.32.160.0/24".parse();
-/// let err = ip6_but_ip6.unwrap_err();
+/// let ip6_but_ip4: Result<Mechanism<IpNetwork>, MechanismError> = "ip6:203.32.160.0/24".parse();
+/// let err = ip6_but_ip4.unwrap_err();
 /// assert_eq!(err, MechanismError::NotIP6Network("203.32.160.0/24".to_string()));
-/// assert_eq!(err.to_string(), "Was given ip6:203.32.160.0/24. This is not an ip6 network.");
+/// assert_eq!(err.to_string(), "203.32.160.0/24 is not an ip6 network");
 ///```
 impl FromStr for Mechanism<IpNetwork> {
     type Err = MechanismError;
@@ -141,26 +155,35 @@ impl FromStr for Mechanism<IpNetwork> {
                 kind = Kind::IpV6;
                 raw_ip = qualifier_and_modified_str.1.strip_prefix("ip6:")
             };
+            // Consider changing this to if let Some() = {}
             let parsed = raw_ip.unwrap().parse();
             if let Ok(parsed_ip) = parsed {
                 let ip: IpNetwork = parsed_ip;
                 if ip.is_ipv4() && kind.is_ip_v4() {
-                    return Ok(Mechanism::new_ip4(qualifier_and_modified_str.0, ip));
+                    return Ok(Mechanism::generic_inclusive(
+                        kind,
+                        qualifier_and_modified_str.0,
+                        Some(ip),
+                    ));
                 } else if ip.is_ipv4() && !kind.is_ip_v4() {
                     return Err(MechanismError::NotIP6Network(ip.to_string()));
                 } else if ip.is_ipv6() && kind.is_ip_v6() {
-                    return Ok(Mechanism::new_ip6(qualifier_and_modified_str.0, ip));
+                    return Ok(Mechanism::generic_inclusive(
+                        kind,
+                        qualifier_and_modified_str.0,
+                        Some(ip),
+                    ));
                 } else if ip.is_ipv6() && !kind.is_ip_v6() {
                     return Err(MechanismError::NotIP4Network(ip.to_string()));
                 };
             } else {
-                return Err(MechanismError::NotValidIPNetwork(
+                return Err(MechanismError::InvalidIPNetwork(
                     parsed.unwrap_err().to_string(),
                 ));
             };
         }
         // Catch all. This is not an ip4 or ip6 spf string.
-        Err(MechanismError::NotValidMechanismFormat(s.to_string()))
+        Err(MechanismError::InvalidMechanismFormat(s.to_string()))
     }
 }
 
@@ -176,13 +199,21 @@ impl<T> Mechanism<T> {
     //! These are the generic methods for the struct of Mechanism.  
     //! All the following methods can be used on any struct of type Mechanism.
     #[doc(hidden)]
-    pub fn new(kind: Kind, qualifier: Qualifier, mechanism: Option<T>) -> Self {
+    pub fn generic_inclusive(kind: Kind, qualifier: Qualifier, mechanism: Option<T>) -> Self {
         Self {
             kind,
             qualifier,
             rrdata: mechanism,
         }
     }
+    #[doc(hidden)]
+    pub fn new(kind: Kind, qualifier: Qualifier) -> Self {
+        Self {
+            kind,
+            qualifier,
+            rrdata: None,
+        }
+    }
     /// Check mechanism is pass
     pub fn is_pass(&self) -> bool {
         self.qualifier == Qualifier::Pass
@@ -216,71 +247,105 @@ impl<T> Mechanism<T> {
 
 impl Mechanism<String> {
     /// Create a new Mechanism struct of `Redirect`
-    #[doc(hidden)]
+    #[deprecated(note = "This will be depreciated in 0.3.0. Please use `redirect()` instead")]
     pub fn new_redirect(qualifier: Qualifier, mechanism: String) -> Self {
-        Mechanism::new(Kind::Redirect, qualifier, Some(mechanism))
+        Mechanism::generic_inclusive(Kind::Redirect, qualifier, Some(mechanism))
     }
 
+    /// Create a new Mechanism struct of `Redirect`
+    pub fn redirect(qualifier: Qualifier, rrdata: &str) -> Result<Self, MechanismError> {
+        Ok(Mechanism::new(Kind::Redirect, qualifier).with_rrdata(rrdata)?)
+    }
     /// Create a new Mechanism struct of `A` with no string value.
-    ///
-    /// # Example:
-    /// ```
-    /// use decon_spf::mechanism::Qualifier;
-    /// use decon_spf::mechanism::Mechanism;
-    /// let mechanism_a = Mechanism::new_a_without_mechanism(Qualifier::Pass);
-    /// assert_eq!(mechanism_a.kind().is_a(), true);
-    /// assert_eq!(mechanism_a.raw(), "a".to_string());
-    /// assert_eq!(mechanism_a.mechanism().is_none(), true);
-    #[doc(hidden)]
+    #[deprecated(note = "This will be depreciated in 0.3.0. Please use `a()` instead")]
     pub fn new_a_without_mechanism(qualifier: Qualifier) -> Self {
-        Mechanism::new(Kind::A, qualifier, None)
+        Mechanism::generic_inclusive(Kind::A, qualifier, None)
     }
 
     /// Create a new Mechanism struct of `A` with string value.
-    ///
-    /// # Example:
-    /// ```rust
-    /// use decon_spf::mechanism::Qualifier;
-    /// use decon_spf::mechanism::Mechanism;
-    /// let mechanism_of_a = Mechanism::new_a_with_mechanism(Qualifier::Pass,
-    ///                                                      String::from("example.com"));
-    /// assert_eq!(mechanism_of_a.qualifier().as_str(), "");
-    /// assert_eq!(mechanism_of_a.raw(), "example.com");
-    /// assert_eq!(mechanism_of_a.to_string(), "a:example.com");
-    /// ```
-    #[doc(hidden)]
+    #[deprecated(
+        note = "This will be depreciated in 0.3.0. Please use `a().with_rrdata()` instead"
+    )]
     pub fn new_a_with_mechanism(qualifier: Qualifier, mechanism: String) -> Self {
-        Mechanism::new(Kind::A, qualifier, Some(mechanism))
+        Mechanism::generic_inclusive(Kind::A, qualifier, Some(mechanism))
     }
 
-    /// Create a new Mechanism struct of `MX` without string value.
+    /// Create a new Mechanism struct of `A`
     ///
     /// # Example:
-    /// ```rust
+    /// ```
     /// use decon_spf::mechanism::Qualifier;
     /// use decon_spf::mechanism::Mechanism;
-    /// let mechanism_mx = Mechanism::new_mx_without_mechanism(Qualifier::Pass);
-    /// assert_eq!(mechanism_mx.kind().is_mx(), true);
-    /// assert_eq!(mechanism_mx.raw(), "mx");
-    /// ```
-    #[doc(hidden)]
+    /// # #[cfg(feature = "strict-dns")]
+    /// # use decon_spf::mechanism::MechanismError;
+    /// // New `A` without rrdata.
+    /// let m = Mechanism::new_a_without_mechanism(Qualifier::Pass);
+    /// assert_eq!(m.kind().is_a(), true);
+    /// assert_eq!(m.raw(), "a".to_string());
+    /// assert_eq!(m.mechanism().is_none(), true);
+    /// // Create `A` with rrdata
+    /// if let Ok(m_with_rrdata) = Mechanism::a(Qualifier::Pass)
+    ///                                                .with_rrdata("example.com") {
+    ///   assert_eq!(m_with_rrdata.raw(), "example.com".to_string());
+    ///   assert_eq!(m_with_rrdata.to_string(), "a:example.com".to_string());
+    /// }
+    /// // Create `A` with bad rrdata and `strict-dns` is disabled
+    /// if let Ok(bad_rrdata) = Mechanism::a(Qualifier::Pass)
+    ///                                             .with_rrdata("example.xx") {
+    ///   assert_eq!(bad_rrdata.raw(), "example.xx".to_string());
+    ///   assert_eq!(bad_rrdata.to_string(), "a:example.xx".to_string());
+    /// }
+    /// // Create `A` with bad rrdata and `strict-dns` is enabled
+    /// # #[cfg(feature = "strict-dns")] {
+    /// if let Err(bad_rrdata) = Mechanism::a(Qualifier::Pass)
+    ///                                              .with_rrdata("example.xx") {
+    ///   assert_eq!(bad_rrdata, MechanismError::InvalidDomainHost("example.xx".to_string()));
+    /// }
+    /// # }
+    ///```
+    pub fn a(qualifier: Qualifier) -> Self {
+        Mechanism::new(Kind::A, qualifier)
+    }
+    /// Create a new Mechanism struct of `MX` without string value.
+    #[deprecated(note = "This will  be depreciated in 0.3.0. Please use `mx()` instead")]
     pub fn new_mx_without_mechanism(qualifier: Qualifier) -> Self {
-        Mechanism::new(Kind::MX, qualifier, None)
+        Mechanism::generic_inclusive(Kind::MX, qualifier, None)
     }
 
     /// Create a new Mechanism struct of `MX`
+    #[deprecated(
+        note = "This will  be depreciated in 0.3.0. Please use `mx().with_rrdata()` instead"
+    )]
+    pub fn new_mx_with_mechanism(qualifier: Qualifier, mechanism: String) -> Self {
+        Mechanism::generic_inclusive(Kind::MX, qualifier, Some(mechanism))
+    }
+
+    /// Create a new Mechanism struct of `MX`
+    ///
     /// # Example:
     /// ```rust
     /// use decon_spf::mechanism::Qualifier;
     /// use decon_spf::mechanism::Mechanism;
-    /// let mechanism_mx = Mechanism::new_mx_with_mechanism(Qualifier::Pass,
-    ///                                                     String::from("example.com"));
-    /// assert_eq!(mechanism_mx.raw(), "example.com");
-    /// assert_eq!(mechanism_mx.to_string(), "mx:example.com")
+    /// // without rrdata
+    /// let mx = Mechanism::mx(Qualifier::Pass);
+    /// assert_eq!(mx.kind().is_mx(), true);
+    /// assert_eq!(mx.raw(), "mx");
+    /// // with rrdata
+    /// if let Ok(mx) = Mechanism::mx(Qualifier::Pass)
+    ///                               .with_rrdata("example.com") {
+    ///   assert_eq!(mx.kind().is_mx(), true);
+    ///   assert_eq!(mx.raw(), "example.com".to_string());
+    ///   assert_eq!(mx.to_string(), "mx:example.com".to_string());
+    /// }
     /// ```
-    #[doc(hidden)]
-    pub fn new_mx_with_mechanism(qualifier: Qualifier, mechanism: String) -> Self {
-        Mechanism::new(Kind::MX, qualifier, Some(mechanism))
+    pub fn mx(qualifier: Qualifier) -> Self {
+        Mechanism::new(Kind::MX, qualifier)
+    }
+
+    /// Create a new Mechanism struct of `Include`
+    #[deprecated(note = "This will  be depreciated in 0.3.0. Please use `include()` instead")]
+    pub fn new_include(qualifier: Qualifier, mechanism: String) -> Self {
+        Mechanism::generic_inclusive(Kind::Include, qualifier, Some(mechanism))
     }
 
     /// Create a new Mechanism struct of `Include`
@@ -288,57 +353,94 @@ impl Mechanism<String> {
     /// ```rust
     /// use decon_spf::mechanism::Qualifier;
     /// use decon_spf::mechanism::Mechanism;
-    /// let mechanism_of_include = Mechanism::new_include(Qualifier::Pass,
-    ///                                                   String::from("example.com"));
-    /// assert_eq!(mechanism_of_include.qualifier().as_str(), "");
-    /// assert_eq!(mechanism_of_include.raw(), "example.com");
-    /// assert_eq!(mechanism_of_include.to_string(), "include:example.com");
-    /// let mechanism_of_include2 = Mechanism::new_include(Qualifier::SoftFail,
-    ///                                                    String::from("example.com"));
-    /// assert_eq!(mechanism_of_include2.to_string(), "~include:example.com")
+    /// let include = Mechanism::include(Qualifier::Pass,
+    ///                                         "example.com").unwrap();
+    /// assert_eq!(include.qualifier().as_str(), "");
+    /// assert_eq!(include.raw(), "example.com");
+    /// assert_eq!(include.to_string(), "include:example.com");
+    /// let include2 = Mechanism::include(Qualifier::SoftFail,
+    ///                                          "example.com").unwrap();
+    /// assert_eq!(include2.to_string(), "~include:example.com")
     /// ```
-    #[doc(hidden)]
-    pub fn new_include(qualifier: Qualifier, mechanism: String) -> Self {
-        Mechanism::new(Kind::Include, qualifier, Some(mechanism))
+    pub fn include(qualifier: Qualifier, rrdata: &str) -> Result<Self, MechanismError> {
+        Ok(Mechanism::new(Kind::Include, qualifier).with_rrdata(rrdata)?)
     }
 
     /// Create a new Mechanism struct of `Ptr` with no value
-    /// # Example:
-    /// ```rust
-    /// use decon_spf::mechanism::Qualifier;
-    /// use decon_spf::mechanism::Mechanism;
-    /// let mechanism_of_ptr = Mechanism::new_ptr_without_mechanism(Qualifier::Fail);
-    /// assert_eq!(mechanism_of_ptr.to_string(), "-ptr");
-    #[doc(hidden)]
+    #[deprecated(note = "This will  be depreciated in 0.3.0. Please use `ptr()` instead")]
     pub fn new_ptr_without_mechanism(qualifier: Qualifier) -> Self {
-        Mechanism::new(Kind::Ptr, qualifier, None)
+        Mechanism::generic_inclusive(Kind::Ptr, qualifier, None)
     }
 
+    /// Create a new Mechanism struct of `Ptr`
+    #[deprecated(
+        note = "This will  be depreciated in 0.3.0. Please use `ptr().with_rrdata()` instead"
+    )]
+    pub fn new_ptr_with_mechanism(qualifier: Qualifier, mechanism: String) -> Self {
+        Mechanism::generic_inclusive(Kind::Ptr, qualifier, Some(mechanism))
+    }
     /// Create a new Mechanism struct of `Ptr`
     /// # Example:
     /// ```rust
     /// use decon_spf::mechanism::Qualifier;
     /// use decon_spf::mechanism::Mechanism;
-    /// let mechanism_of_ptr = Mechanism::new_ptr_with_mechanism(Qualifier::Pass,
-    ///                                                          String::from("example.com"));
-    /// assert_eq!(mechanism_of_ptr.qualifier().as_str(), "");
-    /// assert_eq!(mechanism_of_ptr.raw(), "example.com");
-    /// assert_eq!(mechanism_of_ptr.to_string(), "ptr:example.com");
-    #[doc(hidden)]
-    pub fn new_ptr_with_mechanism(qualifier: Qualifier, mechanism: String) -> Self {
-        Mechanism::new(Kind::Ptr, qualifier, Some(mechanism))
+    /// // without rrdata
+    /// let ptr = Mechanism::ptr(Qualifier::Fail);
+    /// assert_eq!(ptr.to_string(), "-ptr");
+    /// // with rrdata
+    /// let ptr = Mechanism::ptr(Qualifier::Fail)
+    ///                                 .with_rrdata("example.com").unwrap();
+    /// assert_eq!(ptr.to_string(), "-ptr:example.com");
+    /// ```
+    pub fn ptr(qualifier: Qualifier) -> Self {
+        Mechanism::new(Kind::Ptr, qualifier)
     }
 
     /// Create a new Mechanism struct of `Exists`
-    #[doc(hidden)]
+    #[deprecated(
+        note = "This will  be depreciated in 0.3.0. Please use `exists().with_rrdata()` instead"
+    )]
     pub fn new_exists(qualifier: Qualifier, mechanism: String) -> Self {
-        Mechanism::new(Kind::Exists, qualifier, Some(mechanism))
+        Mechanism::generic_inclusive(Kind::Exists, qualifier, Some(mechanism))
+    }
+
+    /// Create a new Mechanism struct of `Exists`
+    pub fn exists(qualifier: Qualifier, rrdata: &str) -> Result<Self, MechanismError> {
+        Ok(Mechanism::new(Kind::Exists, qualifier).with_rrdata(rrdata)?)
+    }
+    /// Set the rrdata for Mechanism
+    /// # Note: This is only applicable for Mechanisms of `A`, `MX` and `Ptr`.  
+    /// All other Mechanism types require `rrdata` to be set. That is to say that `rrdata` is
+    /// **optional** for `A`, `MX` and `PTR`  
+    /// See: [`a`](Mechanism<String>::a) for an example.
+    pub fn with_rrdata(mut self, rrdata: &str) -> Result<Self, MechanismError> {
+        #[cfg(feature = "strict-dns")]
+        {
+            match self.kind() {
+                Kind::A | Kind::MX | Kind::Include | Kind::Ptr | Kind::Exists => {
+                    if !helpers::dns_is_valid(helpers::get_domain_before_slash(&rrdata)) {
+                        return Err(MechanismError::InvalidDomainHost(rrdata.to_string()));
+                    };
+                }
+                _ => {}
+            };
+        }
+        match self.kind() {
+            // Ensure that `All` is always None even if with_rrdata() is called
+            Kind::All => self.rrdata = None,
+            _ => self.rrdata = Some(rrdata.to_string()),
+        }
+        Ok(self)
+    }
+    /// Create a new Mechanism struct of `All`
+    #[deprecated(note = "This will  be depreciated in 0.3.0. Please use `all()` instead")]
+    pub fn new_all(qualifier: Qualifier) -> Self {
+        Mechanism::new(Kind::All, qualifier)
     }
 
     /// Create a new Mechanism struct of `All`
-    #[doc(hidden)]
-    pub fn new_all(qualifier: Qualifier) -> Self {
-        Mechanism::new(Kind::All, qualifier, None)
+    pub fn all(qualifier: Qualifier) -> Self {
+        Mechanism::new(Kind::All, qualifier)
     }
 
     /// Return the mechanism string stored in the `Mechanism`
@@ -360,24 +462,6 @@ impl Mechanism<String> {
         }
     }
 
-    /// Rebuild and return the string representation of the given mechanism
-    ///
-    /// # Example:
-    /// ```
-    /// use decon_spf::mechanism::Qualifier;
-    /// use decon_spf::mechanism::Mechanism;
-    /// let mechanism_a = Mechanism::new_a_without_mechanism(Qualifier::Neutral);
-    /// assert_eq!(mechanism_a.to_string(), "?a");
-    /// let mechanism_a_string = Mechanism::new_a_with_mechanism(Qualifier::Pass,
-    ///                                                          String::from("example.com"));
-    /// assert_eq!(mechanism_a_string.to_string(), "a:example.com");
-    #[deprecated(
-        since = "0.2.0",
-        note = "This will be removed in a future release. Use `to_string()` which is implemented through Display trait."
-    )]
-    pub fn string(&self) -> String {
-        self.build_string()
-    }
     fn build_string(&self) -> String {
         let mut mechanism_str = String::new();
         let tmp_mechanism_str;
@@ -385,15 +469,15 @@ impl Mechanism<String> {
             mechanism_str.push_str(self.qualifier.as_str());
         };
         mechanism_str.push_str(self.kind().as_str());
-        if self.rrdata.is_some() {
-            tmp_mechanism_str = self.rrdata.as_ref().unwrap().as_str();
+        if let Some(ref rrdata) = self.rrdata {
+            tmp_mechanism_str = rrdata.as_str();
         } else {
-            tmp_mechanism_str = ""
+            tmp_mechanism_str = "";
         }
         match self.kind {
             Kind::A | Kind::MX => {
                 // This must be starting with 'domain.com' So prepend ':'
-                if !tmp_mechanism_str.starts_with('/') && !tmp_mechanism_str.is_empty() {
+                if !tmp_mechanism_str.is_empty() && !tmp_mechanism_str.starts_with('/') {
                     mechanism_str.push(':')
                 }
             }
@@ -411,59 +495,98 @@ impl Mechanism<String> {
     }
 }
 
+/// Provide to_string for Mechanism<String>
+
 impl std::fmt::Display for Mechanism<String> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}", self.build_string())
     }
 }
 
+impl From<IpNetworkError> for MechanismError {
+    fn from(err: IpNetworkError) -> Self {
+        MechanismError::InvalidIPNetwork(err.to_string())
+    }
+}
+
 impl Mechanism<IpNetwork> {
+    /// Create a new V4 or V6 Mechanism<IpNetwork>  
+    #[deprecated(note = "This will be depreciated in 0.3.0. Please use `ip()` instead")]
+    pub fn new_ip(qualifier: Qualifier, mechanism: IpNetwork) -> Mechanism<IpNetwork> {
+        Mechanism::ip(qualifier, mechanism)
+    }
+    /// Create a new V4 or V6 Mechanism from a string representation.
+    ///```
+    /// # use decon_spf::mechanism::{Mechanism, MechanismError};
+    /// let string = "+ip4:203.32.160.0/24";
+    /// if let Ok(m) = Mechanism::ip_from_string(&string) {
+    ///   assert_eq!(m.raw(), "203.32.160.0/24");
+    ///   assert_eq!(m.to_string(), "ip4:203.32.160.0/24");
+    /// }
+    ///```
+    pub fn ip_from_string(string: &str) -> Result<Mechanism<IpNetwork>, MechanismError> {
+        Ok(Mechanism::<IpNetwork>::from_str(string)?)
+    }
+
     /// Create a new V4 or V6 Mechanism<IpNetwork>  
     /// Will correctly set its `kind` based on the `IpNetwork` type.
     ///
     /// # Examples:
     /// ```
     /// # use ipnetwork::IpNetwork;
-    /// use decon_spf::mechanism::Qualifier;
-    /// use decon_spf::mechanism::Mechanism;
+    /// use decon_spf::mechanism::{Mechanism, Qualifier};
     ///
     /// // Requires: use ipnetwork::IpNetwork;
     /// let ip: IpNetwork = "192.168.11.0/24".parse().unwrap();
-    /// let mechanism = Mechanism::new_ip(Qualifier::Pass, ip);
+    /// let mechanism = Mechanism::ip(Qualifier::Pass, ip);
     /// assert_eq!(mechanism.kind().is_ip_v4(), true);
     /// assert_eq!(mechanism.raw(), "192.168.11.0/24".to_string());
     /// assert_eq!(mechanism.as_network().to_string(), "192.168.11.0/24".to_string());
     /// assert_eq!(mechanism.as_network().prefix(), 24);
     ///
     /// // This section does not require use of ipnetwork::IpNetwork;
-    /// let mechanism_ip4 = Mechanism::new_ip(Qualifier::Pass,
+    /// let mechanism_ip4 = Mechanism::ip(Qualifier::Pass,
     ///                                       "203.32.160.0/23".parse().unwrap());
     /// assert_eq!(mechanism_ip4.kind().is_ip(), true);
+    /// assert_eq!(mechanism_ip4.kind().is_ip_v4(), true);
     /// assert_eq!(mechanism_ip4.to_string(), "ip4:203.32.160.0/23".to_string());
-    /// let mechanism_ip6 = Mechanism::new_ip(Qualifier::Pass,
+    /// let mechanism_ip6 = Mechanism::ip(Qualifier::Pass,
     ///                                       "2001:4860:4000::/36".parse().unwrap());
     /// assert_eq!(mechanism_ip6.kind().is_ip(), true);
+    /// assert_eq!(mechanism_ip6.kind().is_ip_v6(),true);
     /// assert_eq!(mechanism_ip6.to_string(), "ip6:2001:4860:4000::/36".to_string());
     ///```
-    #[doc(hidden)]
-    pub fn new_ip(qualifier: Qualifier, mechanism: IpNetwork) -> Mechanism<IpNetwork> {
-        if mechanism.is_ipv4() {
-            Mechanism::new_ip4(qualifier, mechanism)
+    pub fn ip(qualifier: Qualifier, rrdata: IpNetwork) -> Mechanism<IpNetwork> {
+        if rrdata.is_ipv4() {
+            Mechanism::ip4(qualifier, rrdata)
         } else {
-            Mechanism::new_ip6(qualifier, mechanism)
+            Mechanism::ip6(qualifier, rrdata)
         }
     }
 
     /// Create a new Mechanism<IpNetwork> of IP4
     #[doc(hidden)]
-    pub fn new_ip4(qualifier: Qualifier, mechanism: IpNetwork) -> Self {
-        Mechanism::new(Kind::IpV4, qualifier, Some(mechanism))
+    #[allow(dead_code)]
+    #[deprecated(note = "This will  be depreciated in 0.3.0. Please use `ip4()` instead")]
+    fn new_ip4(qualifier: Qualifier, mechanism: IpNetwork) -> Self {
+        Mechanism::generic_inclusive(Kind::IpV4, qualifier, Some(mechanism))
+    }
+
+    /// Create a new Mechanism<IpNetwork> of IP4
+    fn ip4(qualifier: Qualifier, rrdata: IpNetwork) -> Self {
+        Mechanism::generic_inclusive(Kind::IpV4, qualifier, Some(rrdata))
+    }
+    /// Create a new Mechanism<IpNetwork> of IP6
+    #[doc(hidden)]
+    #[allow(dead_code)]
+    #[deprecated(note = "This will  be depreciated in 0.3.0. Please use `ip6()` instead")]
+    fn new_ip6(qualifier: Qualifier, mechanism: IpNetwork) -> Self {
+        Mechanism::generic_inclusive(Kind::IpV6, qualifier, Some(mechanism))
     }
 
     /// Create a new Mechanism<IpNetwork> of IP6
-    #[doc(hidden)]
-    pub fn new_ip6(qualifier: Qualifier, mechanism: IpNetwork) -> Self {
-        Mechanism::new(Kind::IpV6, qualifier, Some(mechanism))
+    fn ip6(qualifier: Qualifier, rrdata: IpNetwork) -> Self {
+        Mechanism::generic_inclusive(Kind::IpV6, qualifier, Some(rrdata))
     }
     /// Returns the simple string representation of the mechanism
     /// # Example
@@ -473,7 +596,7 @@ impl Mechanism<IpNetwork> {
     /// use decon_spf::mechanism::Qualifier;
     /// use decon_spf::mechanism::Mechanism;
     /// let ip: IpNetwork = "192.168.11.0/24".parse().unwrap();
-    /// let ip_mechanism = Mechanism::new_ip4(Qualifier::Pass, ip);
+    /// let ip_mechanism = Mechanism::ip(Qualifier::Pass, ip);
     /// assert_eq!(ip_mechanism.raw(), "192.168.11.0/24");
     /// assert_eq!(ip_mechanism.kind().is_ip(), true);
     ///```
@@ -483,26 +606,6 @@ impl Mechanism<IpNetwork> {
         self.rrdata.unwrap().to_string()
     }
 
-    /// Returns the mechanism string representation of an IP4/6 mechanism.
-    /// # Example
-    ///
-    /// ```
-    /// use ipnetwork::IpNetwork;
-    /// use decon_spf::mechanism::Qualifier;
-    /// use decon_spf::mechanism::Mechanism;
-    /// let ip: IpNetwork = "192.168.11.0/24".parse().unwrap();
-    /// let ip_mechanism = Mechanism::new_ip4(Qualifier::Pass, ip);
-    /// assert_eq!(ip_mechanism.to_string(), "ip4:192.168.11.0/24");
-    /// assert_eq!(ip_mechanism.as_network(), &ip);
-    /// ```
-    ///
-    #[deprecated(
-        since = "0.2.0",
-        note = "This will be removed in a future release. Use to_string() which is implemented through Display trait."
-    )]
-    pub fn string(&self) -> String {
-        self.build_string()
-    }
     fn build_string(&self) -> String {
         let mut ip_mechanism_str = String::new();
         if self.qualifier != Qualifier::Pass {
@@ -519,6 +622,7 @@ impl Mechanism<IpNetwork> {
     }
 }
 
+/// Provide to_string for Mechanism<IpNetwork>
 impl std::fmt::Display for Mechanism<IpNetwork> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}", self.build_string())
